;
; PIC12F683 TSTx v1 Beta (26.07.2016) v12
; Copyright (c) 2016 by Wojciech Kokot <wojciech_kokot@wp.eu>
;
    LIST P=12F683, R=DEC
    ; W=2, X=ON, 
    #INCLUDE P12F683.INC
;	__CONFIG 0x0C14 ; = _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
    __CONFIG _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
;			 & _MCLRE_OFF ; MCLR as Pin
;			 & _CP_OFF & _CPD_OFF ; Code/Data Protection Disabled
;			 & _FCMEN_OFF ; Fail-Safe Clock Monitor is disabled

; Constants
LED		Equ 	2	; pin for led
WHI		Equ 	4	; pin for led
BTN		Equ		5	; pin for Button
TBLK	Equ		3 	; Time of Blink
TOFF	Equ		6	; Time of led off

cblock	0x20	; start of general purpose registers
	i			; loop iterator
    mode		; button mode
    count		; number of fast bips
	dxms		; delay x times 10/100ms
	dc_a		; used in delay routine
	dc_b		; used in delay routine
	temp		; temp storage
endc

	ORG 0x0000 ; Reset Vector
	GOTO Start

	ORG 0x0004
;	GOTO Start ; Interupt Vector

Start:
	CALL InitMCU
	CLRF	mode

Off:
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off

;   v10 -> v12
;	SLEEP

WakeUp:	
	CLRF	i
	CLRF	mode
	CALL	Delay250ms

Main:
;   v10 -> v12
;	BTFSC	GPIO, BTN
;	GOTO Off

Check:
	MOVLW	d'10'		; put the value to compare with in W
    SUBWF	i, W		; subtract W from the File Register and Set result in W
	BTFSC   STATUS,C	; if i >= 10
	GOTO Delay		    ; yes

Blink:
	CALL DelayBlink
	BTFSC	i, 0
	GOTO	LedOff
LedOn:
	BSF		GPIO, LED
	BSF		GPIO, WHI	; LEDs ON
	GOTO	Loop
LedOff:
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off
	GOTO	Loop

Delay:
	CALL DelayLong
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off

Loop:
	INCF	i
	BTFSC	i, 4
	CLRF	i

	GOTO Main

SetOn:
	BTFSC	mode, LED 
	GOTO WakeUp

	BCF 	mode, LED
	GOTO Off

SetOff:
	BTFSS	mode, BTN	; 
	GOTO Check

	BSF 	mode, BTN
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off
	CALL	Delay1s
	GOTO Off

InitMCU:
	; Init Inputs/Outputs
	; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	BANKSEL GPIO
	CLRF	GPIO		; Init GPIO
	MOVLW   b'00000111'
	MOVWF   CMCON0		; Disable comparator
	
	BANKSEL	ANSEL
	CLRF	ANSEL		; No Analog
	MOVLW	b'00100000' ; GP5 = BTN, GP4 = LED2, GP3 = Buz, GP2 = PWM/Led
	MOVWF	TRISIO		; Set GP<2-4> as inputs and GP<0,1,5> as outputs
	BSF		IOC, BTN	; BTN (GP5) interupt on-change enabled
	BSF		INTCON, GPIE; GPIO Interupt = Enable

	; Default Clock 4MHz (x110 xxx)
	; Default OSCCON = 0110 1000
	MOVLW	b'00010000' ; = 125kHz
	MOVWF	OSCCON

	BANKSEL GPIO		; Back to bank 0

	RETURN

DelayLong	; At 125KHz (3088 instructions = 98,816ms)
	MOVLW	TOFF	; = 6
	GOTO DLInit
DelayBlink	; At 125KHz (3088 instructions = 98,816ms)
	MOVLW	TBLK	; = 3
	GOTO DLInit
Delay250ms	; At 125KHz (7708 instructions = 246,65ms)
	MOVLW	20
	GOTO DLInit
Delay1s	; At 125kHz (33116 instructions = 1,059s)
	MOVLW	43
DLInit
	MOVWF	dc_b
	CLRF	dc_a
DLoop
	DECFSZ	dc_a, f
	GOTO	DLoop
	DECFSZ	dc_b, f
	GOTO DLoop

	RETURN

	END
