;
; PIC12F683 TSTx v1 Beta (02.08.2019) v13
; Copyright (c) 2016 by Wojciech Kokot <wojciech_kokot@wp.eu>
;

    LIST P=12F683, R=DEC
;

    INCLUDE P12F683.INC

    __CONFIG _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON & _BOD_OFF & _IESO_OFF & _FCMEN_OFF
;			 _INTOSCIO ; Clock I/O pins as GPIO
;			 & _WDT_OFF ; Watchdog Timer is disabled
;			 & _PWRTE_OFF ; Power-up Timer is disabled
;			 & _MCLRE_OFF ; MCLR as GPIO
;			 & _CP_ON & _CPD_ON ; Code/Data Protection enabled
;			 & _BOD_OFF ; Brown-out Reset disabled
;			 & _IESO_OFF ; Internal External Switchove is disabled
;			 & _FCMEN_OFF ; Fail-Safe Clock Monitor is disabled
;	__CONFIG 0x0014 ; Equivalent

; Constants
LED		Equ 	2	; pin for led
WHI		Equ 	4	; pin for led 2
BTN		Equ		5	; pin for Button
TBLK	Equ		3 	; Time of Blink
TOFF	Equ		6	; Time of led off
TORCHON Equ     205 ; Time of torch mode on (~5s)
TORCHPR Equ     39  ; PWM = [39 + 1] x 4 x (1/125e3) x 1 = 1,28ms @ 125kHz
TORCHDC Equ     6   ; Torch duty cycle

cblock	0x20	; start of general purpose registers
	i			; loop iterator
    mode		; button mode
    count		; number of fast bips
	dxms		; delay x times 10/100ms
	dc_a		; used in delay routine
	dc_b		; used in delay routine
	temp		; temp storage
endc

	ORG 0x0000 ; Reset Vector
	GOTO Start

	ORG 0x0004
;	GOTO Start ; Interupt Vector

Start:
	CALL InitMCU
    CALL Torch

Off:
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off

;   v10 -> v12
;	SLEEP

WakeUp:	
	CLRF	i
	CALL	DelayStart

Main:
;   v10 -> v12
;	BTFSC	GPIO, BTN
;	GOTO Off

Check:
	MOVLW	d'10'		; put the value to compare with in W
    SUBWF	i, W		; subtract W from the File Register and Set result in W
	BTFSC   STATUS,C	; if i >= 10
	GOTO Delay		    ; yes

Blink:
	CALL DelayBlink
	BTFSC	i, 0        ; if i & 1
	GOTO	LedOff      ; disable led
LedOn:
	BSF		GPIO, LED
	BSF		GPIO, WHI	; LEDs ON
	GOTO	Loop
LedOff:
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off
	GOTO	Loop

Delay:
	CALL DelayPause
	BCF		GPIO, LED
	BCF		GPIO, WHI	; LEDs off

Loop:
	INCF	i
	BTFSC	i, 4        ; if i > 15
	CLRF	i           ; next period (51175 instructions = 1.6376s @ 125kHz)

	GOTO Main

Torch: ; V13
    ; Set PWM period
	BANKSEL PR2         ; Bank 1
	MOVLW	TORCHPR     ; PWM = [(period) + 1] x 4 x TOSC x TMR2 Prescaler. period from 0 to 255
	MOVWF	PR2			; PWM = [39 + 1] x 4 x (1/125e3) x 1 = 1.28ms @ 125 kHz

	; Set CCP1 as PWM
	BANKSEL CCP1CON     ; Bank 0
	MOVLW	b'00001100'
	MOVWF	CCP1CON		; PWM mode active high

    ; Set Timer2
;   BANKSEL PIR1        ; Bank 0
	BCF		PIR1,TMR2IF ; disable interupt (PWM requirement)
	;MOVLW   b'00000010'
	;MOVWF	T2CON       ; Stop Timer2, Prescaler to 1:4
    CLRF	T2CON       ; Stop Timer2, Prescaler to 1:1
    CLRF	TMR2        ; Clear Timer
	DECF	TMR2        ; Clear Timer

    ; Set PWM duty cycle
;   BANKSEL CCPR1L      ; Bank 0
	MOVLW	TORCHDC
	MOVWF	CCPR1L		; DC = 50

    ; Enable torch (start PWM))
    BSF		T2CON,TMR2ON; Start Timer2

    CALL DelayTorch

    ; Disable torch (stop PWM)
    CLRF	T2CON			; Stop Timer2, Prescaler to 1:1
    CLRF	CCPR1L			; DC = 0
    CLRF	CCP1CON         ; Reset CCP1

    RETURN

InitMCU:
	; Init Inputs/Outputs
	; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	BANKSEL GPIO        ; Bank 0
	CLRF	GPIO		; Init GPIO
	MOVLW   b'00000111'
	MOVWF   CMCON0		; Disable comparator

	BANKSEL	ANSEL       ; Bank 1
	CLRF	ANSEL		; No Analog
	MOVLW	b'00100000'
	MOVWF	TRISIO		; GP2 = PWM/LED1; GP4 = LED2; GP5 = BTN
	;BSF		IOC, BTN	; BTN (GP5) interupt on-change enabled
	;BSF		INTCON, GPIE; GPIO Interupt = Enable

	; CPU Clock (Default: 0110 - 4MHz) / Execution Speed = Clock Speed / 4
	; OSCCON = 0111 - 8MHz; 0110 - 4MHz; ...; 0010 - 250kHz; 0001 - 125kHz; 0000 - 31 kHz
	MOVLW	b'00010000' ; = 125kHz
	MOVWF	OSCCON

	BANKSEL GPIO		; Bank 0

	RETURN

DelayBlink	; 2318 instructions (74.176 ms @ 125 kHz)
	MOVLW	TBLK	; = 3
	GOTO DLInit
DelayPause	; 4628 instructions (148.096 ms @ 125 kHz)
	MOVLW	TOFF	; = 6
	GOTO DLInit
DelayStart	; 15408 instructions (493.056 ms @ 125 kHz)
	MOVLW	20
	GOTO DLInit
DelayTorch	; 157856 instructions (5051.392 @ 125 kHz)
	MOVLW	TORCHON
DLInit ; W * 770 + 8 Instructions (4 * Instructions / Freq [kHz] = Delay [ms])
	MOVWF	dc_b
	CLRF	dc_a
DLoop
	DECFSZ	dc_a, f
	GOTO	DLoop
	DECFSZ	dc_b, f
	GOTO DLoop

	RETURN

	END
