;
; PIC12F683 TSTx v1 Beta
; Copyright (c) 2013 by Wojciech Kokot <wojciech_kokot@wp.eu>
;
    LIST P=12F683, R=DEC
    ; W=2, X=ON, 
    #INCLUDE P12F683.INC
;	__IDLOCS 0x9ABCDEF9 ;!!! Write Device ID
;	__CONFIG 0x0C14 ; = _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
    __CONFIG _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
;			 & _MCLRE_OFF ; MCLR as Pin
;			 & _CP_OFF & _CPD_OFF ; Code/Data Protection Disabled
;			 & _FCMEN_OFF ; Fail-Safe Clock Monitor is disabled

; Constants
LED		Equ 	2	; pin for led
BTN		Equ		5	; pin for Button
BUZ		Equ		0	; pin for Buzzer
SETADDR Equ		0x30; Power settings table address
SETLEN	Equ		2	; Number of settings
BIPDC	Equ		31	; Biping duty cycle

cblock	0x20	; start of general purpose registers
	bip			; how many times to bip
	duty		; duty cycle value (0 - 40)
	dxms		; delay x times 10/100ms
	dc_a		; used in delay routine
	dc_b		; used in delay routine
	temp		; temp storage
endc

cblock	SETADDR	; Power Values
	power: SETLEN ; Array of Power Settings
endc

	ORG 0x0000 ; Reset Vector
	GOTO Start

	ORG 0x0004
;	GOTO Start ; Interupt Vector

Start:
	CALL InitMCU

Main: ; MAIN LOOP
	SLEEP				; Power Off

	BTFSC	GPIO, BTN
	GOTO Main

	; ON
	BSF	GPIO, LED

	CALL Delay200ms

ON:
	BTFSC	GPIO, BTN
	GOTO ON

	; OFF
	BCF	GPIO, LED
	
	CALL Delay200ms

	GOTO Main 			; End of Main loop

InitMCU:
	; Init Inputs/Outputs
	; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	BANKSEL GPIO
	CLRF	GPIO		; Init GPIO
	MOVLW   b'00000111'
	MOVWF   CMCON0		; Disable comparator
	
	BANKSEL	ANSEL
	CLRF	ANSEL		; No Analog
	MOVLW	b'00110000' ; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	MOVWF	TRISIO		; Set GP<2-4> as inputs and GP<0,1,5> as outputs
	BSF		IOC, BTN	; BTN (GP5) interupt on-change enabled
	BSF		INTCON, GPIE; GPIO Interupt = Enable

	; Set 4MHz Clock (x110 xxx)
	; Default OSCCON = 0110 1000
	MOVLW	b'00010000' ; = 250kHz
	MOVWF	OSCCON

	BANKSEL GPIO		; Back to bank 0

	RETURN

Delay20ms	; At 250kHz ( instructions = 10,015ms)
	MOVLW	1
	GOTO DLInit
Delay200ms	; At 250kHz (12332 instructions = 197,2ms)
	MOVLW	21
DLInit
	MOVWF	dc_b
	CLRF	dc_a
DLoop
	DECFSZ	dc_a, f
	GOTO	DLoop
	DECFSZ	dc_b, f
	GOTO DLoop

	RETURN

	END
