;
; PIC12F683 TSTx v1 - On/Off
; Copyright (c) 2013 by Wojciech Kokot <wojciech_kokot@wp.eu>
;
    LIST P=12F683, R=DEC
    ; W=2, X=ON, 
    #INCLUDE P12F683.INC
;	__CONFIG 0x0C14 ; = _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
    __CONFIG _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
;			 & _MCLRE_OFF ; MCLR as Pin
;			 & _CP_OFF & _CPD_OFF ; Code/Data Protection Disabled
;			 & _FCMEN_OFF ; Fail-Safe Clock Monitor is disabled

; Constants
LED		Equ 	2	; pin for led
BTN		Equ		5	; pin for Button
BUZ		Equ		0	; pin for Buzzer

cblock	0x20	; start of general purpose registers
	dxms		; delay x times 10/100ms
	dc_a		; used in delay routine
	dc_b		; used in delay routine
	temp		; temp storage
endc

	ORG 0x0000 ; Reset Vector
	GOTO Start

	ORG 0x0004
;	GOTO Start ; Interupt Vector

Start:
	CALL InitMCU

Main: ; MAIN LOOP
;	SLEEP				; Power Off

	BTFSC	GPIO, BTN
	GOTO Main

	; ON
	BSF	GPIO, LED

	CALL Delay1s

ON:
	BTFSC	GPIO, BTN
	GOTO ON

	; OFF
	BCF	GPIO, LED
	
	CALL Delay1s

	GOTO Main 			; End of Main loop

InitMCU:
	; Init Inputs/Outputs
	; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	BANKSEL GPIO
	CLRF	GPIO		; Init GPIO
	MOVLW   b'00000111'
	MOVWF   CMCON0		; Disable comparator
	
	BANKSEL	ANSEL
	CLRF	ANSEL		; No Analog
	MOVLW	b'00110000' ; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	MOVWF	TRISIO		; Set GP<2-4> as inputs and GP<0,1,5> as outputs
	BSF		IOC, BTN	; BTN (GP5) interupt on-change enabled
	BSF		INTCON, GPIE; GPIO Interupt = Enable

	; Default Clock 4MHz (x110 xxx)
	; Default OSCCON = 0110 1000
	MOVLW	b'00010000' ; = 125kHz
	MOVWF	OSCCON

	BANKSEL GPIO		; Back to bank 0

	RETURN

Delay1s	; At 125kHz (33116 instructions = 1,059s)
	MOVLW	43
DLInit
	MOVWF	dc_b
	CLRF	dc_a
DLoop
	DECFSZ	dc_a, f
	GOTO	DLoop
	DECFSZ	dc_b, f
	GOTO DLoop

	RETURN

	END
