;
; PIC12F683 TSTx v1 Beta (10.01.2015) v05-mod
; Copyright (c) 2013 by Wojciech Kokot <wojciech_kokot@wp.eu>
;
    LIST P=12F683, R=DEC
    ; W=2, X=ON, 
    #INCLUDE P12F683.INC
;	__IDLOCS 0x9ABCDEF9 ;!!! Write Device ID
;	__CONFIG 0x0C14 ; = _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
    __CONFIG _INTOSCIO & _WDT_OFF & _PWRTE_OFF & _BOD_OFF & _MCLRE_OFF & _CP_ON & _CPD_ON
;			 & _MCLRE_OFF ; MCLR as Pin
;			 & _CP_OFF & _CPD_OFF ; Code/Data Protection Disabled
;			 & _FCMEN_OFF ; Fail-Safe Clock Monitor is disabled

; Constants
LED		Equ 	2		; pin for led
BTN		Equ		5		; pin for Button
BUZ		Equ		0		; pin for Buzzer
SETADDR Equ		0x30	; Power settings table address
SETLEN	Equ		4		; Number of settings

; PWM Period/DC
;PWMPR	Equ		d'39'	; PWM = [39 + 1] x 4 x (1/4e6) x 1 = 0,04ms (25kHz)
PWMPR	Equ		d'249'	; PWM = [249 + 1] x 4 x (1/250e3) x 16 = 64ms (15,625Hz)
PWM0	Equ		d'130'	; PWM-DC Start Value
PWMST	Equ		d'60'	; PWM-DC Step

; Biping Period/DC
BIPPR	Equ		d'12'	; PWM = [14 + 1] x 4 x (1/250e3) x 1 = 0,24ms (4,16kHz)
BIPDC	Equ		d'6'	; Biping duty cycle

cblock	0x20	; start of general purpose registers
	bip			; how many times to bip
	duty		; duty cycle value (0 - 40)
	dxms		; delay x times 10/100ms
	dc_a		; used in delay routine
	dc_b		; used in delay routine
	temp		; temp storage
endc

cblock	SETADDR	; Power Values
	power: SETLEN ; Array of Power Settings
endc

	ORG 0x0000 ; Reset Vector
	GOTO Start

	ORG 0x0004
;	GOTO Start ; Interupt Vector

Start:
	CALL InitMCU
	
	SLEEP

Main: ; MAIN LOOP
	BTFSC	GPIO, BTN
	GOTO Main

	CALL SetBipFreq	; Set Low Freq ~ 1.5kHz

	INCF	FSR,F
	BTFSC	FSR, 2		; is next > last?
	GOTO	PwrOff		; Power Off
	
	; Biping Routine
	MOVF	FSR,W
	MOVWF	bip
	MOVLW	SETADDR		; Calc number of Bips
	SUBWF	bip,F		; bip = <1: 3>

BipLoop
	BSF		GPIO, BUZ	; Buz on
	MOVLW	BIPDC
	MOVWF	CCPR1L		; DC = 50
	MOVLW	3
	CALL Delay100x		; 300ms

	BCF		GPIO, BUZ	; Buz off
	CLRF	CCPR1L
	MOVLW	2
	CALL Delay100x		; 200ms
	
	DECFSZ	bip,F
	GOTO BipLoop

	CALL SetWorkFreq		; = 25kHz
	
	MOVF	INDF,W
	MOVWF	CCPR1L		; Update Power Settings

	GOTO Main 			; End of Main loop

PwrOff ; Power Off Routine
	MOVLW	SETADDR
	MOVWF	FSR			; Reset Index
	MOVF	INDF,W
	MOVWF	CCPR1L		; Update Power Settings

	; long biiip :)
	BSF		GPIO, BUZ	; Buz On
	MOVLW	BIPDC
	MOVWF	CCPR1L		; DC = 50

	MOVLW	9
	CALL Delay100x

	BCF		GPIO, BUZ	; Buz Off
	CLRF	CCPR1L

	SLEEP				; Power Off
	GOTO Main			; Back to Main loop

SetBipFreq:
	CLRF	CCPR1L			; DC = 0
	BCF		T2CON, TMR2ON	; Stop PWM
	CLRF	T2CON			; Prescaler to 1:1
;	MOVLW   2
;	MOVWF	T2CON			; Prescaler to 1:4

	BANKSEL PR2
	MOVLW	BIPPR			; PWM = [61 + 1] x 4 x (1/4e6) x 4 = 0,248ms (4,03 kHz)
	MOVWF	PR2				; PWM = [(period) + 1] x 4 x TOSC x TMR2 Prescaler. period from 0 to 255

	BANKSEL TMR2
	CLRF	TMR2
	DECF	TMR2
	BSF		T2CON, TMR2ON	; Run PWM

	RETURN

SetWorkFreq:
	CLRF	CCPR1L			; DC = 0
	BCF		T2CON, TMR2ON	; Stop PWM
;	CLRF	T2CON			; Prescaler to 1:1
	MOVLW   3
	MOVWF	T2CON			; Prescaler to 1:16

	BANKSEL PR2
	MOVLW	PWMPR
	MOVWF	PR2			; PWM = [(period) + 1] x 4 x TOSC x TMR2 Prescaler. period from 0 to 255

	BANKSEL TMR2
	CLRF	TMR2
	DECF	TMR2
	BSF		T2CON, TMR2ON	; Run PWM

	RETURN

InitMCU:
	; Set 4MHz Clock (x110 xxx)
	; Default OSCCON = 0110 1000 = 4MHz
	BANKSEL	OSCCON
	MOVLW	b'00101000'	; = 250kHz
	MOVWF	OSCCON

	; Init Inputs/Outputs
	; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	BANKSEL GPIO
	CLRF	GPIO		; Init GPIO
	MOVLW   b'00000111'
	MOVWF   CMCON0		; Disable comparator
	
	BANKSEL	ANSEL
	MOVLW	b'00001000'
	MOVWF	ANSEL		; GP4/AN3 as Analog In Other as digital I/O
	MOVLW	b'00110000' ; GP5 = BTN, GP4/AN3 = Bat, GP3 = Buz, GP2 = PWM/Led
	MOVWF	TRISIO		; Set GP<2-4> as inputs and GP<0,1,5> as outputs
	BSF		IOC, BTN	; BTN (GP5) interupt on-change enabled
	BSF		INTCON, GPIE; GPIO Interupt = Enable

	; PWM period
	MOVLW	PWMPR		; PWM = [(period) + 1] x 4 x TOSC x TMR2 Prescaler. period from 0 to 255
	MOVWF	PR2			; PWM = [39 + 1] x 4 x (1/4e6) x 1 = 0,04ms (25kHz)

	; Set CCP1 as PWM
	BANKSEL CCP1CON
	MOVF	CCP1CON, W
	ANDLW	h'F0'
	IORLW	b'00001100'
	MOVWF	CCP1CON		; PWM mode active high
	
	BCF		PIR1, TMR2IF; disable interupt
;	MOVF	T2CON, W
;	ANDLW	h'FC'
;	IORLW	h'03'		; Set Prescaler to 16 (xxxx xx1x)
;	MOVWF	T2CON		; Set Prescaler to 1 (xxxx xx00)
;	CLRF	T2CON		; Pre & Postscaler to 1, Timer Off 
	MOVLW   2
	MOVWF	T2CON			; Prescaler to 1:4

	; Init Power Settings Table
	MOVLW	SETADDR
	MOVWF	FSR				
	CLRF	INDF			; PWR[0] = 0
	CLRF	CCPR1L			; Initial Setting = 0%

	; Power Init Loop
	INCF	FSR,F			; 
	MOVLW	PWM0			; W = Start Value (50%)
PwrInit
	MOVWF	INDF			; PWR[Idx] = W
	ADDLW	PWMST			; W += Step (25%)
	INCF	FSR,F
	BTFSS	FSR, 2
	GOTO PwrInit

	MOVLW	SETADDR
	MOVWF	FSR				; Set Idx back to 0

	BSF		T2CON, TMR2ON	; Start the timer2 running
	
	RETURN

Delay10x
	MOVWF	dxms
DL10x
	CALL Delay10ms
	DECFSZ	dxms,F
	GOTO DL10x

	RETURN

Delay100x
	MOVWF	dxms
DL100x
	CALL Delay10ms
;	CALL Delay100ms
	DECFSZ	dxms,F
	GOTO DL100x

	RETURN

Delay10ms	; At 4MHz (10015 instructions = 10,015ms)
	MOVLW	13
	GOTO DLInit
Delay100ms	; At 4MHz (100106 instructions = 100,16ms)
	MOVLW	130
DLInit
	MOVWF	dc_b
	CLRF	dc_a
DLoop
	DECFSZ	dc_a, f
	GOTO	DLoop
	DECFSZ	dc_b, f
	GOTO DLoop

	RETURN

; Unactive
	MOVLW	2
	ADDWF	CCPR1L, W
	MOVWF	duty
	SUBLW	d'42'
	BTFSC	STATUS,Z ; if CCP1RL > 41
	CLRF	duty
	MOVF	duty, W
	MOVWF	CCPR1L

	MOVLW	5
	CALL Delay100x

	GOTO Main

	END
